@@ -402,7 +402,7 @@ class SendMixinTest(unittest.TestCase):
         invoice.Id = 2
         invoice.send(qb=self.qb_client)
 
-        mock_misc_op.assert_called_with("Invoice/2/send", None)
+        mock_misc_op.assert_called_with("invoice/2/send", None, 'application/octet-stream')
 
     @patch('quickbooks.mixins.QuickBooks.misc_operation')
     def test_send_with_send_to_email(self, mock_misc_op):
@@ -410,4 +410,4 @@ class SendMixinTest(unittest.TestCase):
         invoice.Id = 2
         invoice.send(qb=self.qb_client, send_to="test@email.com")
 
-        mock_misc_op.assert_called_with("Invoice/2/send?sendTo=test@email.com", None)
+        mock_misc_op.assert_called_with("invoice/2/send?sendTo=test@email.com", None, 'application/octet-stream')
